/*
 * Copyright LWJGL. All rights reserved.
 * License terms: https://www.lwjgl.org/license
 * MACHINE GENERATED FILE, DO NOT EDIT
 */
package org.lwjgl.nuklear;

import javax.annotation.*;

import java.nio.*;

import org.lwjgl.*;
import org.lwjgl.system.*;

import static org.lwjgl.system.MemoryUtil.*;
import static org.lwjgl.system.MemoryStack.*;

/**
 * Basic string buffer which is only used in context with the text editor to manage and manipulate dynamic or
 * fixed size string content. This is <em>NOT</em> the default string handling method. The only instance you
 * should have any contact with this API is if you interact with an {@link NkTextEdit} object inside one of the copy and
 * paste functions and even there only for more advanced cases.
 * 
 * <h3>Member documentation</h3>
 * 
 * <ul>
 * <li>{@code len} &ndash; in codepoints/runes/glyphs</li>
 * </ul>
 * 
 * <h3>Layout</h3>
 * 
 * <pre><code>
 * struct nk_str {
 *     {@link NkBuffer struct nk_buffer} buffer;
 *     int len;
 * }</code></pre>
 */
@NativeType("struct nk_str")
public class NkStr extends Struct implements NativeResource {

    /** The struct size in bytes. */
    public static final int SIZEOF;

    /** The struct alignment in bytes. */
    public static final int ALIGNOF;

    /** The struct member offsets. */
    public static final int
        BUFFER,
        LEN;

    static {
        Layout layout = __struct(
            __member(NkBuffer.SIZEOF, NkBuffer.ALIGNOF),
            __member(4)
        );

        SIZEOF = layout.getSize();
        ALIGNOF = layout.getAlignment();

        BUFFER = layout.offsetof(0);
        LEN = layout.offsetof(1);
    }

    /**
     * Creates a {@code NkStr} instance at the current position of the specified {@link ByteBuffer} container. Changes to the buffer's content will be
     * visible to the struct instance and vice versa.
     *
     * <p>The created instance holds a strong reference to the container object.</p>
     */
    public NkStr(ByteBuffer container) {
        super(memAddress(container), __checkContainer(container, SIZEOF));
    }

    @Override
    public int sizeof() { return SIZEOF; }

    /** Returns a {@link NkBuffer} view of the {@code buffer} field. */
    @NativeType("struct nk_buffer")
    public NkBuffer buffer() { return nbuffer(address()); }
    /** Returns the value of the {@code len} field. */
    public int len() { return nlen(address()); }

    // -----------------------------------

    /** Returns a new {@code NkStr} instance allocated with {@link MemoryUtil#memAlloc memAlloc}. The instance must be explicitly freed. */
    public static NkStr malloc() {
        return wrap(NkStr.class, nmemAllocChecked(SIZEOF));
    }

    /** Returns a new {@code NkStr} instance allocated with {@link MemoryUtil#memCalloc memCalloc}. The instance must be explicitly freed. */
    public static NkStr calloc() {
        return wrap(NkStr.class, nmemCallocChecked(1, SIZEOF));
    }

    /** Returns a new {@code NkStr} instance allocated with {@link BufferUtils}. */
    public static NkStr create() {
        ByteBuffer container = BufferUtils.createByteBuffer(SIZEOF);
        return wrap(NkStr.class, memAddress(container), container);
    }

    /** Returns a new {@code NkStr} instance for the specified memory address. */
    public static NkStr create(long address) {
        return wrap(NkStr.class, address);
    }

    /** Like {@link #create(long) create}, but returns {@code null} if {@code address} is {@code NULL}. */
    @Nullable
    public static NkStr createSafe(long address) {
        return address == NULL ? null : wrap(NkStr.class, address);
    }

    /**
     * Returns a new {@link NkStr.Buffer} instance allocated with {@link MemoryUtil#memAlloc memAlloc}. The instance must be explicitly freed.
     *
     * @param capacity the buffer capacity
     */
    public static NkStr.Buffer malloc(int capacity) {
        return wrap(Buffer.class, nmemAllocChecked(__checkMalloc(capacity, SIZEOF)), capacity);
    }

    /**
     * Returns a new {@link NkStr.Buffer} instance allocated with {@link MemoryUtil#memCalloc memCalloc}. The instance must be explicitly freed.
     *
     * @param capacity the buffer capacity
     */
    public static NkStr.Buffer calloc(int capacity) {
        return wrap(Buffer.class, nmemCallocChecked(capacity, SIZEOF), capacity);
    }

    /**
     * Returns a new {@link NkStr.Buffer} instance allocated with {@link BufferUtils}.
     *
     * @param capacity the buffer capacity
     */
    public static NkStr.Buffer create(int capacity) {
        ByteBuffer container = __create(capacity, SIZEOF);
        return wrap(Buffer.class, memAddress(container), capacity, container);
    }

    /**
     * Create a {@link NkStr.Buffer} instance at the specified memory.
     *
     * @param address  the memory address
     * @param capacity the buffer capacity
     */
    public static NkStr.Buffer create(long address, int capacity) {
        return wrap(Buffer.class, address, capacity);
    }

    /** Like {@link #create(long, int) create}, but returns {@code null} if {@code address} is {@code NULL}. */
    @Nullable
    public static NkStr.Buffer createSafe(long address, int capacity) {
        return address == NULL ? null : wrap(Buffer.class, address, capacity);
    }

    // -----------------------------------

    /** Returns a new {@code NkStr} instance allocated on the thread-local {@link MemoryStack}. */
    public static NkStr mallocStack() {
        return mallocStack(stackGet());
    }

    /** Returns a new {@code NkStr} instance allocated on the thread-local {@link MemoryStack} and initializes all its bits to zero. */
    public static NkStr callocStack() {
        return callocStack(stackGet());
    }

    /**
     * Returns a new {@code NkStr} instance allocated on the specified {@link MemoryStack}.
     *
     * @param stack the stack from which to allocate
     */
    public static NkStr mallocStack(MemoryStack stack) {
        return wrap(NkStr.class, stack.nmalloc(ALIGNOF, SIZEOF));
    }

    /**
     * Returns a new {@code NkStr} instance allocated on the specified {@link MemoryStack} and initializes all its bits to zero.
     *
     * @param stack the stack from which to allocate
     */
    public static NkStr callocStack(MemoryStack stack) {
        return wrap(NkStr.class, stack.ncalloc(ALIGNOF, 1, SIZEOF));
    }

    /**
     * Returns a new {@link NkStr.Buffer} instance allocated on the thread-local {@link MemoryStack}.
     *
     * @param capacity the buffer capacity
     */
    public static NkStr.Buffer mallocStack(int capacity) {
        return mallocStack(capacity, stackGet());
    }

    /**
     * Returns a new {@link NkStr.Buffer} instance allocated on the thread-local {@link MemoryStack} and initializes all its bits to zero.
     *
     * @param capacity the buffer capacity
     */
    public static NkStr.Buffer callocStack(int capacity) {
        return callocStack(capacity, stackGet());
    }

    /**
     * Returns a new {@link NkStr.Buffer} instance allocated on the specified {@link MemoryStack}.
     *
     * @param stack the stack from which to allocate
     * @param capacity the buffer capacity
     */
    public static NkStr.Buffer mallocStack(int capacity, MemoryStack stack) {
        return wrap(Buffer.class, stack.nmalloc(ALIGNOF, capacity * SIZEOF), capacity);
    }

    /**
     * Returns a new {@link NkStr.Buffer} instance allocated on the specified {@link MemoryStack} and initializes all its bits to zero.
     *
     * @param stack the stack from which to allocate
     * @param capacity the buffer capacity
     */
    public static NkStr.Buffer callocStack(int capacity, MemoryStack stack) {
        return wrap(Buffer.class, stack.ncalloc(ALIGNOF, capacity, SIZEOF), capacity);
    }

    // -----------------------------------

    /** Unsafe version of {@link #buffer}. */
    public static NkBuffer nbuffer(long struct) { return NkBuffer.create(struct + NkStr.BUFFER); }
    /** Unsafe version of {@link #len}. */
    public static int nlen(long struct) { return UNSAFE.getInt(null, struct + NkStr.LEN); }

    // -----------------------------------

    /** An array of {@link NkStr} structs. */
    public static class Buffer extends StructBuffer<NkStr, Buffer> implements NativeResource {

        private static final NkStr ELEMENT_FACTORY = NkStr.create(-1L);

        /**
         * Creates a new {@code NkStr.Buffer} instance backed by the specified container.
         *
         * Changes to the container's content will be visible to the struct buffer instance and vice versa. The two buffers' position, limit, and mark values
         * will be independent. The new buffer's position will be zero, its capacity and its limit will be the number of bytes remaining in this buffer divided
         * by {@link NkStr#SIZEOF}, and its mark will be undefined.
         *
         * <p>The created buffer instance holds a strong reference to the container object.</p>
         */
        public Buffer(ByteBuffer container) {
            super(container, container.remaining() / SIZEOF);
        }

        public Buffer(long address, int cap) {
            super(address, null, -1, 0, cap, cap);
        }

        Buffer(long address, @Nullable ByteBuffer container, int mark, int pos, int lim, int cap) {
            super(address, container, mark, pos, lim, cap);
        }

        @Override
        protected Buffer self() {
            return this;
        }

        @Override
        protected NkStr getElementFactory() {
            return ELEMENT_FACTORY;
        }

        /** Returns a {@link NkBuffer} view of the {@code buffer} field. */
        @NativeType("struct nk_buffer")
        public NkBuffer buffer() { return NkStr.nbuffer(address()); }
        /** Returns the value of the {@code len} field. */
        public int len() { return NkStr.nlen(address()); }

    }

}